@@ -29,6 +29,10 @@ use constant NUMPY_REGEX => qr/
     \Q but this version of numpy is \E (?:0x)?%x
 /xo;
 
+# Guile object files do not objdump/strip correctly, so exclude them
+# from a number of tests. (#918444)
+use constant GUILE_PATH_REGEX => qr,^usr/lib/[^/]+/[^/]+/guile/[^/]+/.+\.go$,o;
+
 # These are the ones file(1) looks for.  The ".zdebug_info" being the
 # compressed version of .debug_info.
 # - Technically, file(1) also looks for .symtab, but that is apparently
@@ -42,7 +46,7 @@ use Lintian::Check qw(check_spelling spelling_tag_emitter);
 use Lintian::Data;
 use Lintian::Relation qw(:constants);
 use Lintian::Tags qw(tag);
-use Lintian::Util qw(internal_error slurp_entire_file strip);
+use Lintian::Util qw(internal_error strip);
 
 my $ARCH_REGEX = Lintian::Data->new('binaries/arch-regex', qr/\s*\~\~/o,
     sub { return qr/$_[1]/ });
@@ -105,10 +109,12 @@ our $HARDENING= Lintian::Data->new('binaries/hardening-tags', qr/\s*\|\|\s*/o,
     \&_split_hash);
 our $HARDENED_FUNCTIONS = Lintian::Data->new('binaries/hardened-functions');
 our $LFS_SYMBOLS = Lintian::Data->new('binaries/lfs-symbols');
+our $OBSOLETE_CRYPT_FUNCTIONS
+  = Lintian::Data->new('binaries/obsolete-crypt-functions', qr/\s*\|\|\s*/o);
 
 our $ARCH_32_REGEX;
 
-sub run {
+sub always {
     my ($pkg, $type, $info, $proc, $group) = @_;
 
     my ($madir, %directories, $built_with_golang, %SONAME);
@@ -133,9 +139,10 @@ sub run {
 
     my $src = $group->get_source_processable;
     if (defined($src)) {
-        $built_with_golang
-          = $src->info->relation('build-depends')
+        $built_with_golang = $src->info->relation('build-depends')
           ->implies('golang-go | golang-any');
+    } else {
+        $built_with_golang = $pkg =~ m/^golang-/;
     }
 
     foreach my $file (sort keys %{$info->objdump_info}) {
@@ -188,6 +195,13 @@ sub run {
                     $has_lfs = 0;
                 }
             }
+
+            if ($foo eq 'UND' and $OBSOLETE_CRYPT_FUNCTIONS->known($sym)) {
+                # Using an obsolete DES encryption function.
+                my $tag = $OBSOLETE_CRYPT_FUNCTIONS->value($sym);
+                tag $tag, $file;
+            }
+
             next if $is_profiled;
             # According to the binutils documentation[1], the profiling symbol
             # can be named "mcount", "_mcount" or even "__mcount".
@@ -284,7 +298,7 @@ sub run {
     }
 
     tag 'package-name-doesnt-match-sonames', "@sonames"
-      if @sonames && !$match_found;
+      if @sonames && !$match_found && $type ne 'udeb';
 
     # process all files in package
     foreach my $file ($info->sorted_index) {
@@ -316,7 +330,7 @@ sub run {
         if ($multiarch eq 'same') {
             unless ($fname
                 =~ m,\b$gnu_triplet_re(?:\b|_)|/(?:$ruby_triplet_re|java-\d+-openjdk-\Q$arch\E|\.build-id)/,
-              ) {
+            ) {
                 tag 'arch-dependent-file-not-in-arch-specific-directory',$file;
             }
         }
@@ -372,6 +386,9 @@ sub run {
             } elsif ($fname =~ m,^usr/lib/debug/\.build-id/,) {
                 # Detached debug symbols could be for a biarch library.
                 $bad = 0;
+            } elsif ($fname =~ GUILE_PATH_REGEX) {
+                # Guile binaries do not objdump/strip (etc.) correctly.
+                $bad = 0;
             } elsif ($ARCH_64BIT_EQUIVS->known($arch)
                 && $fname =~ m,^lib/modules/,) {
                 my $arch64re
@@ -386,7 +403,10 @@ sub run {
             tag 'binary-from-other-architecture', $file if $bad;
         }
 
-        my $strings = slurp_entire_file($info->strings($file));
+        my $stringsfd = $info->strings($file);
+        my $strings = do { local $/; <$stringsfd> };
+        close($stringsfd)
+          or warn "Error closing strings fd: $!";
         my $exceptions = {
             %{ $group->info->spelling_exceptions },
             map { $_ => 1} $BINARY_SPELLING_EXCEPTIONS->all
@@ -404,6 +424,7 @@ sub run {
                 or $pkg =~ m/-dbg$/
                 or $pkg =~ m/debug/
                 or $fname =~ m,/lib/debug/,
+                or $fname =~ GUILE_PATH_REGEX
                 or $fname =~ m,\.gox$,o) {
                 if (    $fileinfo =~ m/executable/
                     and $strings =~ m/^Caml1999X0[0-9][0-9]$/m) {
@@ -443,6 +464,8 @@ sub run {
                   if $rpath
                   =~ m,^/usr/lib/(?:$madir/)?(?:games/)?(?:\Q$pkg\E|\Q$srcpkg\E)(?:/|\z),;
                 next if $rpath =~ m,^\$\{?ORIGIN\}?,;
+                # GHC in Debian uses a scheme for RPATH. (#914873)
+                next if $rpath =~ m,^/usr/lib/ghc/,;
                 next
                   if $directories{$rpath}
                   and $rpath !~ m,^(?:/usr)?/lib(?:/$madir)?/?\z,;
@@ -483,7 +506,7 @@ sub run {
             $fname =~ m,usr/lib/(?:pyshared/)?python2\.\d+/.*(?<!_d)\.so$,
             or(     $fname =~ m,usr/lib/python3/.+\.cpython-\d+([a-z]+)\.so$,
                 and $1 !~ /d/)
-          ) {
+        ) {
             if (index($strings, 'numpy') > -1 and $strings =~ NUMPY_REGEX) {
                 $uses_numpy_c_abi = 1;
             }
@@ -522,6 +545,8 @@ sub run {
                 next if ($fname =~ m%^lib/modules/%);
                 next if ($fname =~ m%^usr/lib/debug/%);
                 next if ($fname =~ m%\.(?:[ce]32|e64)$%);
+                next if ($fname =~ m%^usr/lib/jvm/.*\.debuginfo$%);
+                next if ($fname =~ GUILE_PATH_REGEX);
                 next
                   if (
                     $fname =~ m{
