@@ -48,7 +48,7 @@ my $KNOWN_DBG_PACKAGE = Lintian::Data->new(
 
 my $SIGNING_KEY_FILENAMES = Lintian::Data->new('common/signing-key-filenames');
 
-sub run {
+sub source {
     my ($pkg, undef, $info, undef, $group) = @_;
     my $debian_dir = $info->index_resolved_path('debian/');
     my $dcontrol;
@@ -498,23 +498,6 @@ sub run {
         }
     }
 
-    my @testsuites = split(m/\s*,\s*/, $info->source_field('testsuite', ''));
-
-    if (any { $_ eq 'autopkgtest-pkg-nodejs' } @testsuites) {
-        # Check control file exists in sources
-        my $path = $info->index_resolved_path('debian/tests/pkg-js/test');
-        tag 'pkg-js-autopkgtest-test-is-missing'
-          unless $path and $path->is_open_ok;
-
-        # Ensure all files referenced in debian/tests/pkg-js/files exist
-        $path = $info->index_resolved_path('debian/tests/pkg-js/files');
-        if ($path) {
-            my @list = map { chomp; s/^\s+(.*?)\s+$/$1/; $_ }
-              grep { /\w/ } split /\n/, $path->file_contents;
-            _path_exists($_, $info) foreach (@list);
-        }
-    }
-
     return;
 }
 
@@ -631,44 +614,6 @@ sub check_relation {
     return;
 }
 
-sub _path_exists {
-    my ($expr, $info) = @_;
-
-    # Split each line in path elements
-    my @elem= map { s/\*/.*/g; s/^\.\*$/.*\\w.*/; $_ ? qr{^$_/?$} : () }
-      split m#/#,
-      $expr;
-    my @dir = ('.');
-
-    # Follow directories
-  LOOP: while (my $re = shift @elem) {
-        foreach my $i (0 .. $#dir) {
-            my ($dir, @tmp);
-
-            next unless defined($dir = $info->index_resolved_path($dir[$i]));
-            next unless $dir->is_dir;
-            last LOOP
-              unless (
-                @tmp= map { $_->basename }
-                grep { $_->basename =~ $re } $dir->children
-              );
-
-            # Stop searching: at least one element found
-            return unless @elem;
-
-            # If this is the last element of path, store current elements
-            my $pwd = $dir[$i];
-            $dir[$i] .= '/' . shift(@tmp);
-
-            push @dir, map { "$pwd/$_" } @tmp if @tmp;
-        }
-    }
-
-    # No element found
-    tag 'pkg-js-autopkgtest-file-does-not-exist', $expr;
-    return;
-}
-
 1;
 
 # Local Variables:
