@@ -33,11 +33,10 @@ use List::MoreUtils qw(any true uniq);
 use Lintian::Architecture qw(:all);
 use Lintian::Data ();
 use Lintian::Check qw(check_maintainer);
-use Lintian::Command qw(safe_qx);
 use Lintian::Relation qw(:constants);
 use Lintian::Relation::Version qw(versions_compare);
 use Lintian::Tags qw(tag);
-use Lintian::Util qw($PKGNAME_REGEX $PKGVERSION_REGEX);
+use Lintian::Util qw($PKGNAME_REGEX $PKGVERSION_REGEX safe_qx);
 
 use constant {
     BUILT_USING_REGEX => qr/^$PKGNAME_REGEX \(= $PKGVERSION_REGEX\)$/o,
@@ -254,7 +253,7 @@ our $OBSOLETE_PACKAGES
 our $VIRTUAL_PACKAGES   = Lintian::Data->new('fields/virtual-packages');
 our $SOURCE_FIELDS      = Lintian::Data->new('common/source-fields');
 
-sub run {
+sub always {
     my ($pkg, $type, $info, $proc, $group) = @_;
     my ($version, $arch_indep);
 
@@ -556,7 +555,7 @@ sub run {
             my ($n) = ($base =~ m/^([^_]+)_/o);
 
             if ($source ne $n) {
-                tag 'source-field-does-not-match-pkg-name', "$source != $n";
+                die "Source field does not match package name $source != $n";
             }
             if ($source !~ /^[a-z0-9][-+\.a-z0-9]+\z/) {
                 tag 'source-field-malformed', $source;
@@ -718,6 +717,10 @@ sub run {
         if ($KNOWN_INSECURE_HOMEPAGE_URIS->matches_any($homepage)) {
             tag 'homepage-field-uses-insecure-uri', $orig;
         }
+
+        tag 'homepage-references-obsolete-debian-infrastructure', $orig
+          if $homepage =~ m/\.alioth\.debian\.org/;
+
     } elsif (not $info->native) {
         if ($type eq 'source') {
             my $binary_has_homepage_field = 0;
@@ -1028,6 +1031,12 @@ sub run {
 
     }
 
+    #---- Rules-Requires-Root
+
+    tag 'no-rules-requires-root-field'
+      if defined($info->field('rules-requires-root'))
+      and $type eq 'source';
+
     #---- Package relations (source package)
 
     if ($type eq 'source') {
