@@ -33,7 +33,7 @@ use Lintian::Util qw(normalize_pkg_path $PKGNAME_REGEX);
 our $CLASS_REGEX = qr/\.(?:class|cljc?)/o;
 our $MAX_BYTECODE = Lintian::Data->new('java/constants', qr/\s*=\s*/o);
 
-sub run {
+sub always {
     my ($pkg, $type, $info) = @_;
     my $java_info = $info->java_info;
     my $missing_jarwrapper = 0;
@@ -45,7 +45,8 @@ sub run {
         for my $jar_file (sort keys %{$java_info}) {
             my $files = $java_info->{$jar_file}{files};
             tag 'source-contains-prebuilt-java-object', $jar_file
-              if any { m/$CLASS_REGEX$/i } keys %{$files};
+              if any { m/$CLASS_REGEX$/i } keys %{$files}
+              and $info->name ne 'lintian';
         }
         return;
     }
@@ -98,7 +99,7 @@ sub run {
                 or $mver
                 > $MAX_BYTECODE->value('max-bytecode-existing-version')) {
                 # First public major version was 45 (Java1), latest
-                # version is 54 (Java10).
+                # version is 55 (Java11).
                 tag 'unknown-java-class-version', $jar_file,
                   "($class -> $mver)";
                 # Skip the rest of this Jar.
@@ -149,7 +150,7 @@ sub run {
                         && $jar_file!~m#\.source_[^/]+.jar#
                     )
                     || $cp
-                  ) {
+                ) {
                     tag 'codeless-jar', $jar_file;
                 }
             }
@@ -215,6 +216,7 @@ sub run {
         #     52 -> Java8
         #     53 -> Java9
         #     54 -> Java10
+        #     55 -> Java11
         my $bad = 0;
 
         # If the lowest version used is greater than the requested
