@@ -52,7 +52,7 @@ our %KNOWN_DOCBASE_FORMAT_FIELDS = (
 
 our $SECTIONS = Lintian::Data->new('doc-base/sections');
 
-sub run {
+sub binary {
     my ($pkg, undef, $info, undef, $group) = @_;
     my (%all_files, %all_links);
 
@@ -87,7 +87,7 @@ sub run {
         my $operm = $file->operm;
 
         if ($file->is_file) { # file checks
-            # menu file?
+             # menu file?
             if ($file =~ m,^usr/(lib|share)/menu/\S,o) { # correct permissions?
                 if ($operm & 0111) {
                     tag 'executable-menu-file',
@@ -136,7 +136,7 @@ sub run {
                 $file =~ m{ \A usr/share/doc/(?:[^/]+/)?
                                  (.+\.(?:html|pdf))(?:\.gz)?
                           \Z}xsmo
-              ) {
+            ) {
                 my $name = $1;
                 unless ($name =~ m/^changelog\.html$/o
                     or $name =~ m/^README[.-]/o
@@ -345,13 +345,17 @@ sub check_doc_base_field {
     # classes since otherwise we'd need to deal with wildcards inside
     # character classes and aren't there yet.
     if ($field eq 'index' or $field eq 'files') {
-        my @files = map { split('\s+', $_) } @$vals;
+        my @files = map { split(' ', $_) } @$vals;
 
         if ($field eq 'index' && @files > 1) {
             tag 'doc-base-index-references-multiple-files', "$dbfile:$line";
         }
         for my $file (@files) {
-            next if $file eq '';
+            next if $file =~ m %^/usr/share/doc/%;
+            next if $file =~ m %^/usr/share/info/%;
+            tag 'doc-base-file-references-wrong-path', "$dbfile:$line", $file;
+        }
+        for my $file (@files) {
             my $realfile = delink($file, $all_links);
             # openoffice.org-dev-doc has thousands of files listed so try to
             # use the hash if possible.
@@ -615,7 +619,7 @@ sub check_script {
         if ($interp =~ m,^\#\!\s*(.+),) {
             $interp = $1;
         } else { # hmm, doesn't seem to start with #!
-            # is it a binary? look for ELF header
+             # is it a binary? look for ELF header
             if ($interp =~ m/^\177ELF/) {
                 return; # nothing to do here
             }
@@ -647,7 +651,7 @@ sub check_script {
             m{ (?:^\s*|[;&|]\s*|(?:then|do|exec)\s+)
                (?:\/usr\/bin\/)?update-menus
                (?:\s|[;&|<>]|\Z)}xsm
-          ) {
+        ) {
             # yes, it does.
             $pres->{'calls-updatemenus'} = 1;
 
@@ -674,7 +678,7 @@ sub check_script {
             m{ (?:^\s*|[;&|]\s*|(?:then|do)\s+)
                (?:\/usr\/sbin\/)?install-docs
                (?:\s|[;&|<>]|\Z) }xsm
-          ) {
+        ) {
             # yes, it does.  Does it remove or add a doc?
             if (m/install-docs\s+(?:-r|--remove)\s/) {
                 $pres->{'calls-installdocs-r'} = 1;
