@@ -28,23 +28,26 @@ use strict;
 use warnings;
 use autodie;
 use utf8;
+use v5.16;
 
 use Cwd qw(abs_path);
 use Carp qw(verbose);
 use Getopt::Long();
+use IO::Async::Function;
 use IO::Async::Loop;
 use List::MoreUtils qw(any none);
+use Path::Tiny;
 use POSIX qw(:sys_wait_h);
 use Time::HiRes qw(gettimeofday tv_interval);
 
 my $INIT_ROOT = $ENV{'LINTIAN_ROOT'};
 
-use Lintian::Command qw(safe_qx);
 use Lintian::DepMap;
 use Lintian::DepMap::Properties;
 use Lintian::Data;
 use Lintian::Lab;
 use Lintian::Output qw(:messages);
+use Lintian::Info::Changelog;
 use Lintian::Internal::FrontendUtil qw(
   default_parallel load_collections
   sanitize_environment open_file_or_fd);
@@ -52,7 +55,7 @@ use Lintian::ProcessablePool;
 use Lintian::Profile;
 use Lintian::Tags qw(tag);
 use Lintian::Unpacker;
-use Lintian::Util qw(internal_error parse_boolean strip);
+use Lintian::Util qw(internal_error parse_boolean strip safe_qx);
 
 sanitize_environment();
 
@@ -93,7 +96,7 @@ my ($STATUS_FD, @CLOSE_AT_END, $PROFILE, $TAGS);
 my @certainties = qw(wild-guess possible certain);
 my (@display_level, %display_source, %suppress_tags);
 my ($action, $checks, $check_tags, $dont_check, $received_signal);
-my (@unpack_info, $LAB, %unpack_options, @auto_remove);
+my (@unpack_info, $LAB, %unpack_options);
 my $user_dirs = $ENV{'LINTIAN_ENABLE_USER_DIRS'} // 1;
 my $exit_code = 0;
 
@@ -124,6 +127,7 @@ sub lintian_banner {
 
 sub fatal_error {
     my ($msg) = @_;
+    $msg =~ s/ at .*//;
     print STDERR  "$msg\n";
     exit(2);
 }
@@ -176,8 +180,6 @@ Behavior options:
     --display-source X        restrict displayed tags by source
     -E, --display-experimental display "X:" tags (normally suppressed)
     --no-display-experimental suppress "X:" tags
-    --fail-on-warnings        return a non-zero exit status if warnings found
-                              (Deprecated)
     -i, --info                give detailed info about tags
     -I, --display-info        display "I:" tags (normally suppressed)
     -L, --display-level       display tags with the specified level
@@ -540,7 +542,6 @@ my %opthash = (
     'color=s' => \$opt{'color'},
     'unpack-info|U=s' => \@unpack_info,
     'allow-root' => \$opt{'allow-root'},
-    'fail-on-warnings' => \$opt{'fail-on-warnings'},
     'keep-lab' => \$opt{'keep-lab'},
     'no-tag-display-limit' => sub { $opt{'tag-display-limit'} = 0; },
     'tag-display-limit=i' => \$opt{'tag-display-limit'},
@@ -550,7 +551,7 @@ my %opthash = (
     'no-cfg' => \$opt{'no-cfg'},
     'profile=s' => \$opt{'LINTIAN_PROFILE'},
 
-    'jobs|j:i' => \$opt{'jobs'},
+    'jobs|j=i' => \$opt{'jobs'},
     'ignore-lintian-env' => \$opt{'ignore-lintian-env'},
     'include-dir=s' => \&record_option_too_late,
     'user-dirs!' => \&record_option_too_late,
@@ -612,16 +613,13 @@ sub main {
         if ($opt{'LINTIAN_CFG'}) {
             parse_config_file($opt{'LINTIAN_CFG'});
         }
+        $opt{'LINTIAN_CFG'} //= '';
     }
 
     $ENV{'TMPDIR'} = $opt{'TMPDIR'} if defined($opt{'TMPDIR'});
 
     configure_output();
 
-    if ($opt{'fail-on-warnings'}) {
-        warning('--fail-on-warnings is deprecated');
-    }
-
     # check for arguments
     if (    $action =~ /^(?:check|unpack)$/
         and $#ARGV == -1
@@ -679,8 +677,7 @@ sub main {
     }
 
     @scripts = sort $PROFILE->scripts;
-    $collmap
-      = load_and_select_collections(\@scripts, \@auto_remove,\%unpack_options);
+    $collmap= load_and_select_collections(\@scripts, \%unpack_options);
 
     $opt{'jobs'} = default_parallel() unless defined $opt{'jobs'};
     $unpack_options{'jobs'} = $opt{'jobs'};
@@ -699,21 +696,8 @@ sub main {
     # Now action is always either "check" or "unpack"
     # these two variables are used by process_package
     #  and need to persist between invocations.
-    my $async_loop = IO::Async::Loop->new;
-    $unpacker= Lintian::Unpacker->new($collmap, \%unpack_options);
 
-    if ($action eq 'check') {
-        # Ensure all checks can actually be loaded...
-        foreach my $script (@scripts) {
-            my $cs = $PROFILE->get_script($script);
-            eval {$cs->load_check;};
-            if ($@) {
-                warning("Cannot load check \"$script\"");
-                print STDERR $@;
-                exit 2;
-            }
-        }
-    }
+    $unpacker= Lintian::Unpacker->new($collmap, \%unpack_options);
 
     foreach my $gname (sort $pool->get_group_names) {
         my $success = 1;
@@ -739,14 +723,18 @@ sub main {
                 }
                 $group->clear_cache;
                 if ($exit_code != 2) {
-                    # Double check that no processes are running;
-                    # hopefully it will catch regressions like 3bbcc3b
-                    # earlier.
-                    if (waitpid(-1, WNOHANG) != -1) {
-                        $exit_code = 2;
-                        internal_error(
-                            'Unreaped processes after running checks!?');
-                    }
+                # Double check that no processes are running;
+                # hopefully it will catch regressions like 3bbcc3b
+                # earlier.
+                #
+                # Unfortunately, the cleanup via IO::Async::Function seems keep
+                # a worker unreaped; disabling. Should be revisited.
+                #
+                # if (waitpid(-1, WNOHANG) != -1) {
+                #     $exit_code = 2;
+                #     internal_error(
+                #         'Unreaped processes after running checks!?');
+                # }
                 } else {
                     # If we are interrupted in (e.g.) checks/manpages, it
                     # tends to leave processes behind.  No reason to flag
@@ -756,12 +744,7 @@ sub main {
                 }
                 @group_lpkg = $group->get_processables;
             } else {
-                my @futures;
-                for my $lpkg ($group->get_processables) {
-                    my @pkg_futures = auto_clean_package($lpkg);
-                    push(@futures, @pkg_futures);
-                }
-                $async_loop->await_all(@futures);
+                auto_clean_packages($group);
             }
         };
         my $total_tres = $format_timer_result->($total_raw_res);
@@ -834,6 +817,9 @@ sub main {
         }
     }
 
+    # universal format sorts output from all packages before printing here
+    $Lintian::Output::GLOBAL->print_last();
+
     # }}}
 
     # Wait for any remaining jobs - There will usually not be any
@@ -850,23 +836,45 @@ sub main {
 # Removes all collections with "Auto-Remove: yes"; takes a Lab::Package
 #  - depends on global variables %collection_info
 #
-sub auto_clean_package {
-    my ($lpkg) = @_;
-    my $proc_id = $lpkg->identifier;
+sub auto_clean_packages {
+    my ($group) = @_;
 
-    debug_msg(1, "Auto removing: ${proc_id} ...");
-    my $time = $start_timer->();
+    my $maxjobs = $opt{'jobs'};
+
+    my $removelpkg = IO::Async::Function->new(
+        code => sub {
+            my ($lpkg) = @_;
+
+            my $proc_id = $lpkg->identifier;
+            debug_msg(1, "Auto removing: ${proc_id} ...");
+            my $time = $start_timer->();
+
+            $0 = "Auto removing: ${proc_id}";
+            $lpkg->remove;
 
-    my $future = $lpkg->remove_async;
-    $future->on_ready(
-        sub {
             my $raw_res = $finish_timer->($time);
             debug_msg(1, "Auto removing: ${proc_id} done (${raw_res}s)");
             perf_log("$proc_id,auto-remove entry,${raw_res}");
-            return;
-        });
 
-    return $future;
+            return;
+        },
+        max_workers => $maxjobs,
+        max_worker_calls => 1,
+    );
+
+    my $loop = IO::Async::Loop->new;
+    $loop->add($removelpkg);
+
+    my @serializable = map { $_->clear_cache; $_ } $group->get_processables;
+    my @futures = map { $removelpkg->call(args => [$_]) } @serializable;
+
+    my $done = Future->wait_all(@futures);
+    $done->get;
+
+    $removelpkg->stop;
+    $loop->remove($removelpkg);
+
+    return;
 }
 
 sub post_pkg_process_overrides{
@@ -972,7 +980,6 @@ sub process_group {
     my ($gname, $group) = @_;
     my ($timer, $raw_res, $tres);
     my $all_ok = 1;
-    my $async_loop = IO::Async::Loop->new;
     $timer = $start_timer->();
   PROC:
     foreach my $lpkg ($group->get_processables){
@@ -1020,8 +1027,6 @@ sub process_group {
             my $stats = $TAGS->statistics($lpkg);
             if ($stats->{types}{E}) {
                 $exit_code = 1;
-            } elsif ($opt{'fail-on-warnings'} && $stats->{types}{W}) {
-                $exit_code = 1;
             }
         }
         post_pkg_process_overrides($lpkg);
@@ -1051,19 +1056,12 @@ sub process_group {
         }
     }
 
-    if (@auto_remove) {
+    if (not $opt{'keep-lab'}) {
         # Invoke auto-clean now that the group has been checked
         $timer = $start_timer->();
-        my @futures;
-        foreach my $lpkg ($group->get_processables) {
-            my @pkg_futures = auto_clean_package($lpkg);
-            push(@futures, @pkg_futures);
-        }
-        $async_loop->await_all(@futures);
-        if (any { $_->is_failed } @futures) {
-            $exit_code = 2;
-            $all_ok = 0;
-        }
+
+        auto_clean_packages($group);
+
         $raw_res = $finish_timer->($timer);
         $tres = $format_timer_result->($raw_res);
         debug_msg(1, "Auto-removal all for group $gname done$tres");
@@ -1121,7 +1119,6 @@ sub parse_config_file {
         'display-experimental' => \$opt{'display-experimental'},
         'display-info'         => \&cfg_display_level,
         'display-level'        => \&cfg_display_level,
-        'fail-on-warnings'     => \$opt{'fail-on-warnings'},
         'info'                 => \$opt{'info'},
         'jobs'                 => \$opt{'jobs'},
         'pedantic'             => \&cfg_display_level,
@@ -1129,6 +1126,7 @@ sub parse_config_file {
         'override'             => \&cfg_override,
         'show-overrides'       => \$opt{'show-overrides'},
         'suppress-tags'        => \&record_suppress_tags,
+        'suppress-tags-from-file' => \&record_suppress_tags_from_file,
         'tag-display-limit'    => \$opt{'tag-display-limit'},
         'verbose'              => \&cfg_verbosity,
     );
@@ -1175,11 +1173,6 @@ sub parse_config_file {
                       " - Using the first value!\n";
                     next;
                 }
-                if ($var eq 'fail-on-warnings') {
-                    print STDERR "The config option ${var} is deprecated\n";
-                    print STDERR
-                      " - Found in $opt{'LINTIAN_CFG'} (line: $.)\n";
-                }
                 $conf_opt{$var} = 1;
                 $found = 1;
                 # Translate boolean strings to "0" or "1"; ignore
@@ -1207,11 +1200,11 @@ sub parse_config_file {
 }
 
 sub _find_changes {
-    require Parse::DebianChangelog;
-    my $dch = Parse::DebianChangelog->init(
-        { infile => 'debian/changelog', quiet => 1 });
-    my $data = $dch->data;
-    my $last = $data ? $data->[0] : undef;
+    my $contents = path('debian/changelog')->slurp;
+    my $changelog = Lintian::Info::Changelog->new;
+    $changelog->parse($contents);
+    my @entries = @{$changelog->entries};
+    my $last = @entries ? $entries[0] : undef;
     my ($source, $version);
     my $changes;
     my @archs;
@@ -1221,7 +1214,7 @@ sub _find_changes {
       if exists($ENV{'DEBRELEASE_DEBS_DIR'});
 
     if (not $last) {
-        my @errors = $dch->get_parse_errors;
+        my @errors = @{$changelog->errors};
         if (@errors) {
             print STDERR "Cannot parse debian/changelog due to errors:\n";
             for my $error (@errors) {
@@ -1248,8 +1241,7 @@ sub _find_changes {
     if (exists $ENV{'DEB_BUILD_ARCH'}) {
         push @archs, $ENV{'DEB_BUILD_ARCH'};
     } else {
-        my %opts = ('err' => '&1',);
-        my $arch = safe_qx(\%opts, 'dpkg', '--print-architecture');
+        my $arch = safe_qx('dpkg', '--print-architecture');
         chomp($arch);
         push @archs, $arch if $arch ne '';
     }
@@ -1298,6 +1290,9 @@ sub configure_output {
                 } elsif ($opts{$_} eq 'fullewi') {
                     require Lintian::Output::FullEWI;
                     $Lintian::Output::GLOBAL = Lintian::Output::FullEWI->new;
+                } elsif ($opts{$_} eq 'universal') {
+                    require Lintian::Output::Universal;
+                    $Lintian::Output::GLOBAL = Lintian::Output::Universal->new;
                 }
             }
         }
@@ -1385,6 +1380,8 @@ sub configure_output {
     } else {
         open($STATUS_FD, '>', '/dev/null');
     }
+
+    $Lintian::Output::GLOBAL->print_first();
     return;
 }
 
@@ -1420,7 +1417,8 @@ sub setup_work_pool {
 }
 
 sub load_profile_and_configure_tags {
-    my $profile = dplint::load_profile($opt{'LINTIAN_PROFILE'});
+    my $profile = eval { dplint::load_profile($opt{'LINTIAN_PROFILE'}); };
+    fatal_error($@) if $@;
     # Ensure $opt{'LINTIAN_PROFILE'} is defined
     $opt{'LINTIAN_PROFILE'} = $profile->name
       unless defined($opt{'LINTIAN_PROFILE'});
@@ -1433,6 +1431,9 @@ sub load_profile_and_configure_tags {
     $TAGS->sources(keys(%display_source)) if %display_source;
     $TAGS->profile($profile);
 
+    undef $checks
+      if ($checks // q{})  eq 'all';
+
     if ($dont_check || %suppress_tags || $checks || $check_tags) {
         _update_profile($profile, $TAGS, $dont_check, \%suppress_tags,$checks);
     }
@@ -1440,17 +1441,13 @@ sub load_profile_and_configure_tags {
     # Initialize display level settings.
     for my $level (@display_level) {
         eval { $TAGS->display(@{$level}) };
-        if ($@) {
-            my $error = $@;
-            $error =~ s/ at .*//;
-            fatal_error($error);
-        }
+        fatal_error($@) if $@;
     }
     return $profile;
 }
 
 sub load_and_select_collections {
-    my ($all_checks, $auto_remove_list, $unpack_options_ref) = @_;
+    my ($all_checks, $unpack_options_ref) = @_;
     # $map is just here to check that all the needed collections are present.
     my $map = Lintian::DepMap->new;
     my $collmap = Lintian::DepMap::Properties->new;
@@ -1461,7 +1458,6 @@ sub load_and_select_collections {
         debug_msg(2, "Read collector description for $coll ...");
         $collmap->add($coll, $cs->needs_info, $cs);
         $map->addp('coll-' . $coll, 'coll-', $cs->needs_info);
-        push(@{$auto_remove_list}, $coll) if $cs->auto_remove;
     };
 
     load_collections($load_coll, "$INIT_ROOT/collection");
@@ -1469,6 +1465,7 @@ sub load_and_select_collections {
     for my $c (@{$all_checks}) {
         # Add the checks with their dependency information
         my $cs = $PROFILE->get_script($c);
+        die "Cannot find check $c" unless defined $cs;
         my @deps = $cs->needs_info;
         $map->add('check-' . $c);
         if (@deps) {
@@ -1520,13 +1517,7 @@ sub load_and_select_collections {
             }
             $extra_unpack{$i} = 1;
         }
-        # Never auto-remove anything explicitly requested by the user
-        @{$auto_remove_list}
-          = grep { !exists($extra_unpack{$_}) } @{$auto_remove_list}
-          if not $opt{'keep-lab'};
     }
-    # Never auto-remove anything if keep-lab is given...
-    @{$auto_remove_list} = () if $opt{'keep-lab'};
     return $collmap;
 }
 
