@@ -25,11 +25,13 @@ use warnings;
 use autodie;
 use parent 'Lintian::Collect::Package';
 
-use Lintian::Relation;
 use Carp qw(croak);
-use Parse::DebianChangelog;
+use Path::Tiny;
 
-use Lintian::Util qw(fail open_gz parse_dpkg_control get_file_checksum strip);
+use Lintian::Deb822Parser qw(parse_dpkg_control);
+use Lintian::Info::Changelog;
+use Lintian::Relation;
+use Lintian::Util qw(internal_error open_gz get_file_checksum strip);
 
 =head1 NAME
 
@@ -142,8 +144,9 @@ sub changelog {
             $changelog = $shared->{'changelog'}{$checksum};
         }
         if (not $changelog) {
-            my %opts = (infile => $dch, quiet => 1);
-            $changelog = Parse::DebianChangelog->init(\%opts);
+            my $contents = path($dch)->slurp;
+            $changelog = Lintian::Info::Changelog->new;
+            $changelog->parse($contents);
             if (defined($shared)) {
                 $shared->{'changelog'}{$checksum} = $changelog;
             }
@@ -342,7 +345,7 @@ sub scripts {
             chomp($line);
 
             $line =~ m/^(env )?(\S*) (.*)$/o
-              or fail("bad line in scripts file: $line");
+              or internal_error("bad line in scripts file: $line");
             ($file{calls_env}, $file{interpreter}, $name) = ($1, $2, $3);
 
             $name =~ s,^\./,,o;
